H264Source

Struct H264Source 

pub struct H264Source<H: SeekableH264Source> {
    pub h264_metadata: Option<H264Metadata>,
    /* private fields */
}
H264 data source. Can come directly from an “Annex B” format .h264 file or from an MP4 file.

This should be as general purpose H264 file reader as possible.

Strand Camera specific features are supported if present: metadata at the H264 stream start (UUID 0ba99cc7-f607-3851-b35e-8c7d8c04da0a) is parsed, as are precision time stamps (specified by MISB ST 0604.3).

Timestamp handling:

Case 1, H264 in MP4

Tracks in MP4 files are composed of samples. Each sample has a presentation timestamp (PTS), the time elapsed from the start. A sample contains multiple (0, 1, 2 or more) H264 NAL units. A sample can contain zero, one, two or more image frames of data. (There exist MP4 files in which a sample contains zero NAL units and thus zero image frames as well as MP4 files in which a single sample contains many NAL units and many image frames.) Samples also carry a duration, which is informational to assist with playback. The duration of frame N should be the PTS of frame N+1 - the PTS of frame N. There seems to be a general assumption that samples should be equi-distant in time and thus that the file has a constant frame rate, although I have not found this in any specification.

Case 2, raw H264

A raw .h264 file, which is defined as the “Annex B format”, (or simply the H264 data inside an MP4 file) may have no explicit timing information in it. Alternatively, for example with the timing_info_present_flag in the VUI parameters, timing information may be present. So far we ignore these flags. In this case, the timestamp data for the frame is simply returned as a fraction of complete (in the interval from 0.0 to 1.0).

Case 3, H264 files with MISPmicrosectime supplemental enhancement information

H264 can contain additional NAL units, called supplemental enhancement information (SEI) which is ignored by decoders but can provide additional information such as metadata at the start of H264 data and per-frame timestamps as specified in MISB ST 0604.3.

Fields

h264_metadata: Option<H264Metadata>

Implementations

Source

impl<H: SeekableH264Source> H264Source<H>

pub fn as_seekable_h264_source(&self) -> &H

pub fn mp4_sample_timing(&self) -> Option<&[Mp4SampleTiming]>

Per-sample timing (decode duration + composition offset) for MP4 sources, indexed by frame number (see FrameData::idx). None for non-MP4 sources.

pub fn srt_truncation(&self) -> Option<&SrtTruncation>

Set when an SRT-file timestamp source ran out of usable stanzas before covering every frame in the container; None otherwise.